@@ -60,7 +60,7 @@ static struct argp_option options[] = {
      { "include", 'i', N_("CLASSES"), 0,
        N_("include specified classes of symbols (see below); prepend CLASSES with ^ or - to exclude them from the output"), GROUP_ID+1 },
      { "format", 'f', N_("NAME"), 0,
-       N_("use given output format NAME; valid names are `gnu' (default) and `posix'"),
+       N_("use given output format NAME; valid names are `gnu' (default), `posix', and `dot'"),
        GROUP_ID+1 },
      { "reverse", 'r', NULL, 0,
        N_("print reverse call tree"), GROUP_ID+1 },
@@ -272,10 +272,24 @@ symbol_override(const char *str)
 	  error(EX_USAGE, 0, _("%s: no symbol type supplied"), str);
      else {
 	  name = strndup(str, ptr - str);
+	  if (!name)
+	       xalloc_die();
 	  if (ptr[1] == '=') {
-	       Symbol *alias = lookup(ptr+2);
-	       if (!alias) {
-		    alias = install(xstrdup(ptr+2), INSTALL_OVERWRITE);
+	       Symbol *alias;
+
+	       ptr += 2;
+	       if (strcmp(name, ptr) == 0) {
+		    error(EX_USAGE, 0, _("cyclic alias: %s -> %s"), name, ptr);
+	       }
+	       
+	       alias = lookup(ptr);
+	       if (alias) {
+		    if (strcmp(alias->name, name) == 0) {
+			 error(EX_USAGE, 0, _("cyclic alias: %s -> %s -> %s"),
+			       name, ptr, alias->name);
+		    }
+	       } else {
+		    alias = install(xstrdup(ptr), INSTALL_OVERWRITE);
 		    alias->type = SymToken;
 		    alias->token_type = 0;
 		    alias->source = NULL;
@@ -794,6 +808,7 @@ main(int argc, char **argv)
      
      register_output("gnu", gnu_output_handler, NULL);
      register_output("posix", posix_output_handler, NULL);
+     register_output("dot", dot_output_handler, NULL);
 
      symbol_map = SM_FUNCTIONS|SM_STATIC|SM_UNDEFINED;
 
