@@ -1,5 +1,5 @@
 /* This file is part of GNU cflow
-   Copyright (C) 1997-2019 Sergey Poznyakoff
+   Copyright (C) 1997-2021 Sergey Poznyakoff
 
    GNU cflow is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
 
 static Hash_table *symbol_table;
 
+static struct linked_list *start_symbol_list;
+static struct linked_list *target_symbol_list;
 static struct linked_list *static_symbol_list;
 static struct linked_list *auto_symbol_list;
 static struct linked_list *static_func_list;
@@ -145,17 +147,13 @@ ident_change_storage(Symbol *sp, enum storage storage)
      default:
 	  break;
      }
+     
      sp->storage = storage;
 }
 
-Symbol *
-install_ident(char *name, enum storage storage)
+void
+init_ident(Symbol *sp, enum storage storage)
 {
-     Symbol *sp;
-
-     sp = install(name, 
-                  storage != AutoStorage ? 
-                     INSTALL_CHECK_LOCAL : INSTALL_DEFAULT);
      sp->type = SymIdentifier;
      sp->arity = -1;
      sp->storage = ExternStorage;
@@ -166,6 +164,17 @@ install_ident(char *name, enum storage storage)
      sp->caller = sp->callee = NULL;
      sp->level = -1;
      ident_change_storage(sp, storage);
+}
+
+Symbol *
+install_ident(char *name, enum storage storage)
+{
+     Symbol *sp;
+
+     sp = install(name, 
+                  storage != AutoStorage ? 
+                     INSTALL_CHECK_LOCAL : INSTALL_DEFAULT);
+     init_ident(sp, storage);
      return sp;
 }
 
@@ -243,6 +252,7 @@ static_free(void *data)
 void
 delete_statics()
 {
+     reset_static_caller();
      if (static_symbol_list) {
 	  static_symbol_list->free_data = static_free;
 	  linked_list_destroy(&static_symbol_list);
@@ -407,4 +417,109 @@ move_parms(int level)
 	  }
      }
 }
+
+Symbol *
+install_starter(char *name)
+{
+     Symbol *sp = install(name, 0);
+     sp->flag = symbol_start;
+     if (!data_in_list(sp, start_symbol_list))
+	  linked_list_append(&start_symbol_list, sp);
+     return sp;
+}
+
+void
+set_default_starter(void)
+{
+     if (!linked_list_head(start_symbol_list))
+	  install_starter("main");
+}
+
+void
+clear_starters(void)
+{
+     linked_list_destroy(&start_symbol_list);
+}
+
+Symbol *
+first_starter(void *itr)
+{
+     struct linked_list_entry *p = linked_list_head(start_symbol_list);
+     while (p) {
+	  Symbol *sym = p->data;
+	  if (sym->type != SymUndefined) {
+	       *(void**)itr = p->next;
+	       return sym;
+	  }
+	  p = p->next;
+     }
+     *(void**)itr = NULL;
+     return NULL;
+}
+
+Symbol *
+next_starter(void *itr)
+{
+     struct linked_list_entry *p;
+
+     if (!itr)
+	  return NULL;
+     p = *(void**)itr;
+     while (p) {
+	  Symbol *sym = p->data;
+	  if (sym->type != SymUndefined) {
+	       *(void**)itr = p->next;
+	       return sym;
+	  }
+	  p = p->next;
+     }
+     *(void**)itr = NULL;
+     return NULL;
+}
+
+Symbol *
+install_target(char *name)
+{
+     Symbol *sp = install(name, 0);
+     sp->flag = symbol_target;
+     if (!data_in_list(sp, target_symbol_list))
+	  linked_list_append(&target_symbol_list, sp);
+     return sp;
+}
+
+static void
+mark_callers(Symbol *sym)
+{
+     struct linked_list_entry *p;
+
+     if (sym->active)
+	  return;
+     sym->active = 1;
+     for (p = linked_list_head(sym->caller); p; p = p->next) {
+	  mark_callers((Symbol*)p->data);
+     }
+     sym->visible = 1;
+     sym->active = 0;
+}
+
+/* Eliminate all paths not leading to selected targets */
+void
+eliminate_non_targets(void)
+{
+     struct linked_list_entry *p;
+     
+     p = linked_list_head(target_symbol_list);
+     
+     if (!p)
+	  return;
+     
+     for (; p; p = p->next) {
+	  Symbol *sym = p->data;
+	  if (sym->flag == symbol_target) {
+	       mark_callers(sym);
+	  }
+     }
+     output_visible = 1;
+}
+
 
